@@ -40,7 +40,7 @@ class Detection(BoundingBox):
         self.attributes = {} if attributes is None else copy.copy(attributes)  # shallow copy
 
     @classmethod
-    def cast(cls, d, flush=False):
+    def cast(cls, d, flush=False, category=None):
         assert isinstance(d, BoundingBox)
         if d.__class__ != Detection:
             d.__class__ = Detection
@@ -50,6 +50,8 @@ class Detection(BoundingBox):
             d._confidence = None if flush or not hasattr(d, '_confidence') else d._confidence
             d._label = None if flush or not hasattr(d, '_label') else d._label
             d.attributes = {} if flush or not hasattr(d, 'attributes') else d.attributes
+            if category is not None:
+                d._label = category  # when casting BoundingBox to Detection, extra args are necessary
         return d
         
     @classmethod
@@ -245,7 +247,7 @@ class Track(object):
         """The confidence of a track is the mean confidence of all (or just last=last frames, or samples=samples uniformly spaced) keyboxes (if confidences are available) else 0"""
         if samples is not None:
             dt = max(1, int(round(len(self._keyframes)/float(samples))))
-            C = [self._keyboxes[i]._confidence for i in range(len(self._keyframes)-1, 0, -dt) if (hasattr(self._keyboxes[i], '_confidence') and self._keyboxes[i]._confidence is not None)]
+            C = [self._keyboxes[i]._confidence for i in range(len(self._keyframes)-1, -1, -dt) if (hasattr(self._keyboxes[i], '_confidence') and self._keyboxes[i]._confidence is not None)]
         elif last == 1:
             return self.endbox().confidence() if len(self)>0 else 0
         else:
@@ -362,9 +364,9 @@ class Track(object):
         """
         assert len(self._keyboxes) > 0, "Degenerate object for interpolation"   # not self.isempty()
         if len(self._keyboxes) == 1:
-            return Detection.cast(self._keyboxes[0].clone()).setattribute('trackid', self.id()) if (self._boundary == 'extend' or self.during(f)) else None
+            return Detection.cast(self._keyboxes[0].clone(), category=self.category()).setattribute('trackid', self.id()) if (self._boundary == 'extend' or self.during(f)) else None
         if f in reversed(self._keyframes):            
-            return Detection.cast(self._keyboxes[self._keyframes.index(f)]).setattribute('trackid', self.id())
+            return Detection.cast(self._keyboxes[self._keyframes.index(f)], category=self.category()).setattribute('trackid', self.id())
 
         kf = self._keyframes
         ft = min(max(f, kf[0]), kf[-1])  # truncated frame index
@@ -485,10 +487,16 @@ class Track(object):
             self._id = newid
             return self
 
-    def clone(self):
+    def clone(self, startframe=None, endframe=None):
         #return copy.deepcopy(self)  
-        return Track.from_json(self.json(encode=False))  # 2x faster than deepcopy
+        return Track.from_json(self.json(encode=False)) if (startframe is None and endframe is None) else self.clone_during(startframe, endframe)  # 2x faster than deepcopy
 
+    def clone_during(self, startframe, endframe):
+        #return copy.deepcopy(self)
+        kfkb = [(kf,kb.clone()) for (kf,kb) in zip(self._keyframes, self._keyboxes) if ((startframe is None or kf >= startframe) and (endframe is None or kf <= endframe))]
+        (kf, kb) = zip(*kfkb) if len(kfkb) > 0 else ([], [])        
+        return Track(keyframes=kf, boxes=kb, category=self._label, framerate=self._framerate, interpolation=self._interpolation, boundary=self._boundary, shortlabel=self._shortlabel, attributes=self.attributes, trackid=self._id)
+    
     def boundingbox(self, startframe=None, endframe=None):
         """The bounding box of a track is the smallest spatial box that contains all of the detections within startframe and endframe, or None if there are no detections"""
         t = self.clone() if (startframe is None and endframe is None) else self.clone().truncate(startframe, endframe)
@@ -732,7 +740,7 @@ class Track(object):
         """
         if self.during(k):
             return self[k]
-        elif len(self.keyboxes()) == 1:
+        elif len(self._keyboxes) == 1:
             return self.nearest_keybox(k)
         else:
             n = self.endframe() if k > self.endframe() else self.startframe()+1
